/*
 *  Instance.cpp
 *  Assignment9
 *
 *  Created by Zach Gildersleeve on 4/8/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Instance.h"

Instance::Instance()
{
	x = y = z = 0.0;
	xAngle = yAngle = zAngle = 0.0;
	xScale = yScale = zScale = 0.0;
}

Instance::Instance(Primitive *newObject)
{
	object = newObject;
	x = y = z = 0.0;
	xAngle = yAngle = zAngle = 0.0;
	xScale = yScale = zScale = 0.0;
}

Instance::Instance(Primitive *newObject,
				   double newX, double newY, double newZ,
				   double newXangle, double newYangle, double newZangle,
				   double newXscale, double newYscale, double newZscale)
{
	object = newObject;
	x = newX;
	y = newY;
	z = newZ;
	xAngle = newXangle;
	yAngle = newYangle;
	zAngle = newZangle;
	xScale = newXscale;
	yScale = newYscale;
	zScale = newZscale;
}

void Instance::setPosition( double newX, double newY, double newZ)
{
	x = newX;
	y = newY;
	z = newZ;
}
void Instance::setRotation(double newXangle, double newYangle, double newZangle)
{	
	xAngle = newXangle * convert;
	yAngle = newYangle * convert;
	zAngle = newZangle * convert;
}
void Instance::setScale(double newXscale, double newYscale, double newZscale)
{
	xScale = newXscale;
	yScale = newYscale;
	zScale = newZscale;
}

void Instance::preprocess()
{
	Matrix translationMatrix;
	Matrix rotationMatrix;
	Matrix rotateXMatrix;
	Matrix rotateYMatrix;
	Matrix rotateZMatrix;
	Matrix scaleMatrix;
	
	translationMatrix = translate(x, y, z);
	rotateXMatrix = rotateX(xAngle);
	rotateYMatrix = rotateY(yAngle);
	rotateZMatrix = rotateZ(zAngle);
	rotationMatrix = rotateXMatrix * rotateYMatrix * rotateZMatrix;
	
	scaleMatrix = scale(xScale, yScale, zScale);
	
	transMat = translationMatrix * scaleMatrix * rotationMatrix;
	invTransMat = transMat.getInverse();
//	std::cout << transMat;
}


void Instance::getTextCoord(Vector &textCoord, const RenderContext &context, const Ray &ray, const HitRecord &hit) const
{
	object->getTextCoord(textCoord, context, ray, hit);
}

bool Instance::intersect(HitRecord &hit, const RenderContext &context, const Ray &ray) const
{
	//printf("in instance intersect\n");
	Vector transformedDirection = transformVector(invTransMat, ray.direction);
//	transformedDirection.normalize();
	Point transformedOrigin = transformPoint(invTransMat, ray.origin);
	Ray transformedRay(transformedOrigin, transformedDirection);
	
	if (object->intersect(hit, context, transformedRay))
	{
		currentPrimitive = hit.getPrimitive();
		return true;
	}
	
	return false;
}

void Instance::normal(Vector &normal, const Point &hitPoint, const HitRecord &hit,
					  const RenderContext &context, const Ray &ray) const
{
	Vector n;
	currentPrimitive->normal(n, hitPoint, hit, context, ray);
	
	n = transformVector(invTransMat, n);
	//n.normalize();
	normal = n;
}

void Instance::getBounds(BoundingBox& bbox) const
{
	Point tempPoint = object->bbox.getMin();
	tempPoint = transformPoint(invTransMat, tempPoint);
	object->bbox.setMin(tempPoint);
	tempPoint = object->bbox.getMax();
	tempPoint = transformPoint(invTransMat, tempPoint);
	object->bbox.setMax(tempPoint);
}
